@@ -72,7 +72,7 @@
  */
 
 typedef int (*bg_read_audio_func_t)(void * priv, gavl_audio_frame_t* frame, int stream,
-                                  int num_samples);
+                                    int num_samples);
 
 /** \ingroup plugin
  *  \brief Generic prototype for reading video
@@ -131,9 +131,7 @@ typedef int (*bg_read_video_func_t)(void * priv, gavl_video_frame_t* frame, int
 /** @}
  */
 
-
-
-#define BG_PLUGIN_API_VERSION 16
+#define BG_PLUGIN_API_VERSION 19
 
 /* Include this into all plugin modules exactly once
    to let the plugin loader obtain the API version */
@@ -680,6 +678,17 @@ struct bg_input_plugin_s
 
   bg_read_audio_func_t read_audio;
 
+  /** \brief Check is a still image is available
+   *  \param priv The handle returned by the create() method
+   *  \param stream Stream index starting with 0
+   *  \returns 1 if a still frame can be decoded, 0 else.
+   *
+   *  If EOF occurs in still streams, this function will return
+   *  1, but the subsequent call to read_video returns 0
+   */
+
+  int (*has_still)(void * priv, int stream);
+  
   /** \brief Read a video frame
    *  \param priv The handle returned by the create() method
    *  \param frame The frame, where the image will be copied
@@ -767,7 +776,7 @@ struct bg_input_plugin_s
   /** \brief Seek within a media track
    *  \param priv The handle returned by the create() method
    *  \param time Time to seek to
-   *  \param scale Scale in which time is given
+   *  \param scale Timescale
    *  
    *  Media streams are supposed to be seekable, if this
    *  function is non-NULL AND the duration field of the track info
@@ -1296,8 +1305,8 @@ struct bg_encoder_plugin_s
    */
   
   int (*open)(void * data, const char * filename,
-              bg_metadata_t * metadata, bg_chapter_list_t * chapter_list);
-
+              const bg_metadata_t * metadata, const bg_chapter_list_t * chapter_list);
+  
   /** \brief Return the filename, which can be passed to the player
    *  \param priv The handle returned by the create() method
    *
@@ -1728,7 +1737,17 @@ struct bg_image_reader_plugin_s
   
   int (*read_header)(void * priv, const char * filename,
                      gavl_video_format_t * format);
-
+  
+  const bg_metadata_t * (*get_metadata)(void * priv);
+  
+  /** \brief Get file infos
+   *  \param priv The handle returned by the create() method
+   *  \returns 
+   */
+  
+  
+  char ** (*get_info)(void * priv);
+  
   /** \brief Read the image
    *  \param priv The handle returned by the create() method
    *  \param frame The frame, where the image will be copied
@@ -1754,6 +1773,7 @@ typedef struct bg_image_writer_plugin_s bg_image_writer_plugin_t;
 struct bg_image_writer_plugin_s
   {
   bg_plugin_common_t common; //!< Infos and functions common to all plugin types
+  const char * extensions; //!< Supported file extensions (space separated)
 
   /** \brief Return the file extension
    *  \param priv The handle returned by the create() method
@@ -1777,7 +1797,7 @@ struct bg_image_writer_plugin_s
    */
   
   int (*write_header)(void * priv, const char * filename,
-                      gavl_video_format_t * format);
+                      gavl_video_format_t * format, const bg_metadata_t * m);
   
   /** \brief Write the image
    *  \param priv The handle returned by the create() method
@@ -1911,6 +1931,16 @@ struct bg_fv_plugin_s
   {
   bg_plugin_common_t common; //!< Infos and functions common to all plugin types
 
+  /** \brief Get gavl options
+   *  \param priv The handle returned by the create() method
+   *  \returns Video conversion options
+   *
+   *  This optional function returns the gavl options. You can configure them
+   *  like you do it in plain gavl.
+   */
+  
+  gavl_video_options_t * (*get_options)(void * priv);
+  
   /** \brief Set input callback
    *  \param priv The handle returned by the create() method
    *  \param func The function to call
@@ -2069,7 +2099,7 @@ struct bg_visualization_plugin_s
    *  \param priv The handle returned by the create() method
    *
    *  This function is needed only for OpenGL plugins. Under X11,
-   *  it will typically call glXSwapBuffers and process eventy on the
+   *  it will typically call glXSwapBuffers and process events on the
    *  X11 window.
    */
   
