@@ -296,6 +296,7 @@ void bg_album_insert_xml_before(bg_album_t * a,
   {
   bg_album_entry_t * new_entries;
   bg_album_entry_t * current_entry;
+  
   new_entries = load_album_xml(a,
                                xml_string, (bg_album_entry_t**)0, &current_entry, 0);
   bg_album_insert_entries_before(a, new_entries, after);
@@ -554,9 +555,10 @@ static xmlDocPtr album_2_xml(bg_album_t * a)
   return xml_doc;
   }
 
-static xmlDocPtr entries_2_xml(bg_album_entry_t * entry, int preserve_current,
+static xmlDocPtr entries_2_xml(bg_album_t * a, int preserve_current,
                                int selected)
   {
+  bg_album_entry_t * entry;
   xmlDocPtr  xml_doc;
   xmlNodePtr xml_album;
   
@@ -568,10 +570,12 @@ static xmlDocPtr entries_2_xml(bg_album_entry_t * entry, int preserve_current,
 
   xmlAddChild(xml_album, BG_XML_NEW_TEXT("\n"));
 
+  entry = a->entries;
+  
   while(entry)
     {
     if((entry->flags & BG_ALBUM_ENTRY_SELECTED) || !selected)
-      save_entry(NULL, entry, xml_album, preserve_current);
+      save_entry(a, entry, xml_album, preserve_current);
     entry = entry->next;
     }
   return xml_doc;
@@ -647,29 +651,7 @@ char * bg_album_save_selected_to_memory(bg_album_t * a, int preserve_current)
   
   memset(&ctx, 0, sizeof(ctx));
   
-  xml_doc = entries_2_xml(a->entries, preserve_current, 1);
-  
-  b = xmlOutputBufferCreateIO(bg_xml_write_callback,
-                              bg_xml_close_callback,
-                              &ctx,
-                              (xmlCharEncodingHandlerPtr)0);
-  
-  xmlSaveFileTo(b,
-                xml_doc,
-                (const char*)0);
-  xmlFreeDoc(xml_doc);
-  return ctx.buffer;
-  }
-
-char * bg_album_entries_save_to_memory(bg_album_entry_t * e)
-  {
-  xmlDocPtr  xml_doc;
-  bg_xml_output_mem_t ctx;
-  xmlOutputBufferPtr b;
-  
-  memset(&ctx, 0, sizeof(ctx));
-  
-  xml_doc = entries_2_xml(e, 0, 0);
+  xml_doc = entries_2_xml(a, preserve_current, 1);
   
   b = xmlOutputBufferCreateIO(bg_xml_write_callback,
                               bg_xml_close_callback,
