@@ -78,7 +78,8 @@ static GtkTargetList * target_list_r = (GtkTargetList *)0;
 static const GtkTargetEntry dnd_src_entries[] = 
   {
     { bg_gtk_atom_entries_name, 0, DND_GMERLIN_TRACKS },
-    { "STRING",          0, DND_TEXT_PLAIN     },
+    { "STRING",          0, DND_TEXT_PLAIN            },
+    { "text/plain",      0, DND_TEXT_PLAIN            },
   };
 
 static const GtkTargetEntry dnd_src_entries_r[] = 
@@ -839,7 +840,9 @@ static void bg_gtk_album_widget_update_entry(bg_gtk_album_widget_t * w, const bg
   bg_album_entry_t * current_entry;
   current_entry = bg_album_get_current_entry(w->album);
   update_entry(w, entry, iter, (current_entry == entry));
-  bg_gtk_time_display_update(w->total_time, bg_album_get_duration(w->album));
+  bg_gtk_time_display_update(w->total_time,
+                             bg_album_get_duration(w->album),
+                             BG_GTK_DISPLAY_MODE_HMS);
   }
 
 static void bg_gtk_album_widget_update(bg_gtk_album_widget_t * w)
@@ -905,7 +908,9 @@ static void bg_gtk_album_widget_update(bg_gtk_album_widget_t * w)
       }
     }
 
-  bg_gtk_time_display_update(w->total_time, bg_album_get_duration(w->album));
+  bg_gtk_time_display_update(w->total_time,
+                             bg_album_get_duration(w->album),
+                             BG_GTK_DISPLAY_MODE_HMS);
 
   set_sensitive(w);
     
@@ -2051,6 +2056,15 @@ static void drag_get_callback(GtkWidget *widget,
     return;
     }
   target_atom = gdk_atom_intern("STRING", FALSE);
+  if(target_atom == data->target)
+    {
+    str = bg_album_selected_to_string(w->album);
+    gtk_selection_data_set(data, type_atom, 8, (uint8_t*)str, strlen(str));
+    free(str);
+    w->drag_delete = 0; 
+    return;
+    }
+  target_atom = gdk_atom_intern("text/plain", FALSE);
   if(target_atom == data->target)
     {
     str = bg_album_selected_to_string(w->album);
@@ -2503,7 +2517,8 @@ static void delete_callback(bg_album_t * a,
   update_indices(w, indices[0]);
   update_cursor_pos(w);
   bg_gtk_time_display_update(w->total_time,
-                             bg_album_get_duration(w->album));
+                             bg_album_get_duration(w->album),
+                             BG_GTK_DISPLAY_MODE_HMS);
   set_sensitive(w);
   find_widget_reset(w);
   }
@@ -2909,7 +2924,7 @@ bg_gtk_album_widget_create(bg_album_t * album, GtkWidget * parent)
   ret->find_button               = create_pixmap_button(ret, "find_16.png",
                                                         TRS("Find tracks"));
   
-  ret->total_time                = bg_gtk_time_display_create(BG_GTK_DISPLAY_SIZE_SMALL, 4);
+  ret->total_time                = bg_gtk_time_display_create(BG_GTK_DISPLAY_SIZE_SMALL, 4, BG_GTK_DISPLAY_MODE_HMS);
 
   bg_gtk_tooltips_set_tip(bg_gtk_time_display_get_widget(ret->total_time),
                           TRS("Total playback time"),
@@ -2945,7 +2960,7 @@ bg_gtk_album_widget_create(bg_album_t * album, GtkWidget * parent)
   
   ret->widget = gtk_vbox_new(0, 0);
 
-  gtk_box_pack_start_defaults(GTK_BOX(ret->widget), scrolledwin);
+  bg_gtk_box_pack_start_defaults(GTK_BOX(ret->widget), scrolledwin);
   gtk_box_pack_start(GTK_BOX(ret->widget), ret->find_widget.box, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(ret->widget), ret->toolbar, FALSE, FALSE, 0);
   
@@ -3280,7 +3295,7 @@ static void find_widget_run(bg_gtk_album_widget_t * w)
     gtk_box_pack_start(GTK_BOX(w->find_widget.box), w->find_widget.close_button,
                        FALSE, FALSE, 0);
 
-    gtk_box_pack_start_defaults(GTK_BOX(w->find_widget.box), w->find_widget.entry);
+    bg_gtk_box_pack_start_defaults(GTK_BOX(w->find_widget.box), w->find_widget.entry);
     
     gtk_box_pack_start(GTK_BOX(w->find_widget.box), w->find_widget.down_button,
                        FALSE, FALSE, 0);
