@@ -596,10 +596,10 @@ static gboolean idle_callback(gpointer data)
         set_line_index(w, PATH_AUDIO, tmp_string, 1);
         free(tmp_string);
         
-        bg_msg_get_arg_audio_format(msg, 1, &arg_af);
+        bg_msg_get_arg_audio_format(msg, 1, &arg_af, NULL);
         set_audio_format(w, PATH_AUDIO_FORMAT_I, &arg_af);
 
-        bg_msg_get_arg_audio_format(msg, 2, &arg_af);
+        bg_msg_get_arg_audio_format(msg, 2, &arg_af, NULL);
         set_audio_format(w, PATH_AUDIO_FORMAT_O, &arg_af);
         
         break;
@@ -612,10 +612,10 @@ static gboolean idle_callback(gpointer data)
         set_line_index(w, PATH_SUBTITLE, tmp_string, 1);
         free(tmp_string);
         
-        bg_msg_get_arg_video_format(msg, 2, &arg_vf);
+        bg_msg_get_arg_video_format(msg, 2, &arg_vf, NULL);
         set_video_format(w, PATH_SUBTITLE_FORMAT_I, &arg_vf);
 
-        bg_msg_get_arg_video_format(msg, 3, &arg_vf);
+        bg_msg_get_arg_video_format(msg, 3, &arg_vf, NULL);
         set_video_format(w, PATH_SUBTITLE_FORMAT_O, &arg_vf);
         
         break;
@@ -626,10 +626,10 @@ static gboolean idle_callback(gpointer data)
         set_line_index(w, PATH_VIDEO, tmp_string, 1);
         free(tmp_string);
         
-        bg_msg_get_arg_video_format(msg, 1, &arg_vf);
+        bg_msg_get_arg_video_format(msg, 1, &arg_vf, NULL);
         set_video_format(w, PATH_VIDEO_FORMAT_I, &arg_vf);
 
-        bg_msg_get_arg_video_format(msg, 2, &arg_vf);
+        bg_msg_get_arg_video_format(msg, 2, &arg_vf, NULL);
         set_video_format(w, PATH_VIDEO_FORMAT_O, &arg_vf);
         
         break;
@@ -1200,12 +1200,15 @@ void bg_gtk_info_window_destroy(bg_gtk_info_window_t * w)
 void bg_gtk_info_window_show(bg_gtk_info_window_t * w)
   {
   if(!w->width || !w->height)
+    {
     gtk_window_set_position(GTK_WINDOW(w->window), GTK_WIN_POS_CENTER);
-  
+    }
   gtk_widget_show(w->window);
   if(w->width && w->height)
     gtk_decorated_window_move_resize_window(GTK_WINDOW(w->window),
                                             w->x, w->y, w->width, w->height);
+  else
+    gtk_window_resize(GTK_WINDOW(w->window), 400, 400);
   }
 
 void bg_gtk_info_window_hide(bg_gtk_info_window_t * w)
