@@ -152,6 +152,7 @@ void bg_player_input_create(bg_player_t * player)
   {
   bg_player_input_context_t * ctx;
   ctx = calloc(1, sizeof(*ctx));
+  player->input_context = ctx;
 
   /* Set up callbacks */
 
@@ -166,7 +167,6 @@ void bg_player_input_create(bg_player_t * player)
   
   ctx->player = player;
   
-  player->input_context = ctx;
   pthread_mutex_init(&(player->input_context->config_mutex),(pthread_mutexattr_t *)0);
   }
 
@@ -538,9 +538,8 @@ bg_player_input_read_audio(void * priv, gavl_audio_frame_t * frame, int stream,
   ctx->audio_samples_written += frame->valid_samples;
 #ifdef DEBUG_COUNTER
   ctx->audio_sample_counter += frame->valid_samples;
+  //  fprintf(stderr, "Got %d of %d samples, result: %d\n", frame->valid_samples, samples, result);
 #endif
-
-  
   return result;
   }
 
@@ -558,19 +557,37 @@ bg_player_input_read_video(void * priv, gavl_video_frame_t * frame, int stream)
     if(!ctx->still_frame)
       {
       ctx->still_frame = gavl_video_frame_create(format);
-      bg_plugin_lock(ctx->plugin_handle);
-      result = ctx->plugin->read_video(ctx->priv, ctx->still_frame,
-                                             stream);
-      bg_plugin_unlock(ctx->plugin_handle);
       ctx->still_frame->timestamp = 0;
-      if(!result)
-        return 0;
       }
+
+    bg_plugin_lock(ctx->plugin_handle);
+
+    if(ctx->plugin->has_still(ctx->priv, stream))
+      result = ctx->plugin->read_video(ctx->priv, frame,
+                                       stream);
     
-    gavl_video_frame_copy(format, frame, ctx->still_frame);
-    frame->timestamp = ctx->still_frame->timestamp;
+    else
+      result = 0;
+    
+    bg_plugin_unlock(ctx->plugin_handle);
+    
+    if(result)
+      gavl_video_frame_copy(format, ctx->still_frame, frame);
+    else
+      {
+      gavl_video_frame_copy(format, frame, ctx->still_frame);
+      frame->timestamp = ctx->still_frame->timestamp;
+      }
     ctx->still_frame->timestamp += format->frame_duration;
     result = 1;
+#ifdef DUMP_TIMESTAMPS
+    bg_debug("Input timestamp: %"PRId64"\n",
+             gavl_time_unscale(ctx->player->video_stream.input_format.timescale,
+                               frame->timestamp));
+#endif
+
+    
+
     }
   else
     {
@@ -591,7 +608,9 @@ bg_player_input_read_video(void * priv, gavl_video_frame_t * frame, int stream)
   }
 
 int
-bg_player_input_read_video_subtitle_only(void * priv, gavl_video_frame_t * frame, int stream)
+bg_player_input_read_video_subtitle_only(void * priv,
+                                         gavl_video_frame_t * frame,
+                                         int stream)
   {
   bg_player_input_context_t * ctx;
   bg_player_video_stream_t * s;
@@ -616,7 +635,11 @@ static int process_audio(bg_player_input_context_t * ctx, int preload)
   bg_player_audio_stream_t * s;
   bg_fifo_state_t state;
   s = &(ctx->player->audio_stream);
-
+#if 0
+  fprintf(stderr, "process_audio A: %f, V: %f\n",
+          gavl_time_to_seconds(ctx->audio_time),
+          gavl_time_to_seconds(ctx->video_time));
+#endif
   if(ctx->send_silence)
     {
     if(preload)
@@ -649,8 +672,9 @@ static int process_audio(bg_player_input_context_t * ctx, int preload)
   
   if(!s->in_func(s->in_data, audio_frame, s->in_stream,
                  s->fifo_format.samples_per_frame))
+    {
     ctx->audio_finished = 1;
-  
+    }
   if(ctx->audio_finished &&
      (!ctx->video_finished || !ctx->subtitle_finished))
     ctx->send_silence = 1;
@@ -757,7 +781,11 @@ static int process_video(bg_player_input_context_t * ctx, int preload)
   gavl_video_frame_t * video_frame;
   bg_player_video_stream_t * s;
   s = &(ctx->player->video_stream);
-
+#if 0
+  fprintf(stderr, "process_video A: %f, V: %f\n",
+          gavl_time_to_seconds(ctx->audio_time),
+          gavl_time_to_seconds(ctx->video_time));
+#endif  
   if(preload || DO_SUBTITLE_ONLY(ctx->player->flags))
     video_frame = (gavl_video_frame_t*)bg_fifo_try_lock_write(s->fifo,
                                                               &state);
@@ -776,9 +804,10 @@ static int process_video(bg_player_input_context_t * ctx, int preload)
     {
     ctx->video_frames_written++;
     }
-  if(ctx->player->video_stream.input_format.framerate_mode != GAVL_FRAMERATE_STILL)
-    ctx->video_time = gavl_time_unscale(ctx->player->video_stream.input_format.timescale,
-                                        video_frame->timestamp);
+  //  if(ctx->player->video_stream.input_format.framerate_mode != GAVL_FRAMERATE_STILL)
+    ctx->video_time =
+      gavl_time_unscale(ctx->player->video_stream.output_format.timescale,
+                        video_frame->timestamp);
   
   bg_fifo_unlock_write(s->fifo, ctx->video_finished);
   
@@ -787,8 +816,6 @@ static int process_video(bg_player_input_context_t * ctx, int preload)
 
 void * bg_player_input_thread(void * data)
   {
-  char tmp_string[128];
-  
   bg_player_input_context_t * ctx;
   bg_msg_t * msg;
   bg_fifo_state_t state;
@@ -868,6 +895,7 @@ void * bg_player_input_thread(void * data)
   
   bg_msg_queue_unlock_write(ctx->player->command_queue);
 
+#if 0  
   if(DO_AUDIO(ctx->player->flags))
     {
     sprintf(tmp_string, "%" PRId64, ctx->audio_samples_written);
@@ -880,6 +908,7 @@ void * bg_player_input_thread(void * data)
     bg_log(BG_LOG_DEBUG, LOG_DOMAIN, "Processed %s video frames",
            tmp_string);
     }
+#endif
   return NULL;
   }
 
@@ -970,11 +999,16 @@ void bg_player_input_seek(bg_player_input_context_t * ctx,
                           ctx->player->video_stream.output_format.frame_duration,
                           ctx->video_time);
   else
+    {
     ctx->video_frames_written =
       gavl_time_to_frames(ctx->player->video_stream.input_format.timescale,
                           ctx->player->video_stream.input_format.frame_duration,
                           ctx->video_time);
-  
+    if(ctx->still_frame)
+      ctx->still_frame->timestamp =
+        gavl_time_scale(ctx->player->video_stream.input_format.timescale,
+                        *time);
+    }
   // Clear EOF states
   do_audio = DO_AUDIO(ctx->player->flags);
   do_video = DO_VIDEO(ctx->player->flags);
