@@ -348,7 +348,7 @@ static int start_input(void * priv)
                                        inp->filename_buffer,
                                        &(inp->track_info.video_streams[0].format)))
       return 0;
-    inp->track_info.video_streams[0].format.timescale = 0;
+    inp->track_info.video_streams[0].format.timescale = GAVL_TIME_SCALE;
     inp->track_info.video_streams[0].format.frame_duration = 0;
     inp->track_info.video_streams[0].format.framerate_mode = GAVL_FRAMERATE_STILL;
     }
@@ -369,6 +369,11 @@ static int start_input(void * priv)
   return 1;
   }
 
+static int has_frame_input(void * priv, int stream)
+  {
+  return 1;
+  }
+                           
 static int read_video_frame_input(void * priv, gavl_video_frame_t* f,
                                   int stream)
   {
@@ -544,6 +549,9 @@ static const bg_input_plugin_t input_plugin_stills =
      *  in the stream infos to check out, which streams are to be decoded
      */
     .start =                 start_input,
+
+    .has_still =             has_frame_input,
+    
     /* Read one video frame (returns FALSE on EOF) */
     .read_video =      read_video_frame_input,
     /*
@@ -669,6 +677,8 @@ typedef struct
   bg_plugin_handle_t * plugin_handle;
   bg_image_writer_plugin_t * image_writer;
   
+  bg_metadata_t metadata;
+  
   bg_parameter_info_t * parameters;
 
   bg_plugin_registry_t * plugin_reg;
@@ -781,8 +791,8 @@ static const char * get_extension_encoder(void * data)
 
 
 static int open_encoder(void * data, const char * filename,
-                        bg_metadata_t * metadata,
-                        bg_chapter_list_t * chapter_list)
+                        const bg_metadata_t * metadata,
+                        const bg_chapter_list_t * chapter_list)
   {
   encoder_t * e;
   int filename_len;
@@ -806,6 +816,9 @@ static int open_encoder(void * data, const char * filename,
 
   e->first_filename = bg_sprintf(e->mask, e->frame_counter);
 
+  if(metadata)
+    bg_metadata_copy(&e->metadata, metadata);
+  
   return 1;
   }
 
@@ -827,8 +840,8 @@ static int write_frame_header(encoder_t * e)
   
   ret = e->image_writer->write_header(e->plugin_handle->priv,
                                       e->filename_buffer,
-                                      &(e->format));
-
+                                      &(e->format), &e->metadata);
+  
   if(!ret)
     {
     e->have_header = 0;
@@ -910,6 +923,8 @@ static int close_encoder(void * data, int do_delete)
   STR_FREE(e->filename_buffer);
   STR_FREE(e->first_filename);
   
+  bg_metadata_free(&e->metadata);
+  
   if(e->plugin_handle)
     {
     bg_plugin_unref(e->plugin_handle);
