@@ -138,8 +138,8 @@ static const char * get_extension_lqt(void * data)
   }
 
 static int open_lqt(void * data, const char * filename,
-                    bg_metadata_t * metadata,
-                    bg_chapter_list_t * chapter_list)
+                    const bg_metadata_t * metadata,
+                    const bg_chapter_list_t * chapter_list)
   {
   char * track_string;
   e_lqt_t * e = (e_lqt_t*)data;
@@ -161,27 +161,30 @@ static int open_lqt(void * data, const char * filename,
     
   /* Set metadata */
 
-  if(metadata->copyright)
-    quicktime_set_copyright(e->file, metadata->copyright);
-  if(metadata->title)
-    quicktime_set_name(e->file, metadata->title);
-
-  if(metadata->comment)
-    lqt_set_comment(e->file, metadata->comment);
-  if(metadata->artist)
-    lqt_set_artist(e->file, metadata->artist);
-  if(metadata->genre)
-    lqt_set_genre(e->file, metadata->genre);
-  if(metadata->track)
+  if(metadata)
     {
-    track_string = bg_sprintf("%d", metadata->track);
-    lqt_set_track(e->file, track_string);
-    free(track_string);
+    if(metadata->copyright)
+      quicktime_set_copyright(e->file, metadata->copyright);
+    if(metadata->title)
+      quicktime_set_name(e->file, metadata->title);
+
+    if(metadata->comment)
+      lqt_set_comment(e->file, metadata->comment);
+    if(metadata->artist)
+      lqt_set_artist(e->file, metadata->artist);
+    if(metadata->genre)
+      lqt_set_genre(e->file, metadata->genre);
+    if(metadata->track)
+      {
+      track_string = bg_sprintf("%d", metadata->track);
+      lqt_set_track(e->file, track_string);
+      free(track_string);
+      }
+    if(metadata->album)
+      lqt_set_album(e->file, metadata->album);
+    if(metadata->author)
+      lqt_set_author(e->file, metadata->author);
     }
-  if(metadata->album)
-    lqt_set_album(e->file, metadata->album);
-  if(metadata->author)
-    lqt_set_author(e->file, metadata->author);
   
   e->chapter_list = chapter_list;
   
@@ -242,13 +245,6 @@ static int add_video_stream_lqt(void * data,
   
   gavl_video_format_copy(&(e->video_streams[e->num_video_streams].format),
                          format);
-
-#if 0  
-  /* AVIs are made with constant framerates only */
-  if((e->file_type & (LQT_FILE_AVI|LQT_FILE_AVI_ODML)))
-    e->video_streams[e->num_video_streams].format.framerate_mode =
-      GAVL_FRAMERATE_CONSTANT;
-#endif // Done by lqtgavl
   
   e->num_video_streams++;
   return e->num_video_streams-1;
@@ -394,29 +390,42 @@ static int write_subtitle_text_lqt(void * data,const char * text,
 static int close_lqt(void * data, int do_delete)
   {
   int i;
+  gavl_time_t chapter_time;
   char * filename_final, *pos;
+  int num_chapters;
   e_lqt_t * e = (e_lqt_t*)data;
-
+  
   if(!e->file)
     return 1;
-
+  
   if(!(e->file_type & (LQT_FILE_AVI|LQT_FILE_AVI_ODML)) &&
      e->chapter_list)
     {
-    /* Write chapter information */
+    /* Count the chapters actually written */
+    
+    num_chapters = 0;
     for(i = 0; i < e->chapter_list->num_chapters; i++)
       {
-      if(e->chapter_list->chapters[i].time > e->duration)
+      chapter_time = gavl_time_unscale(e->chapter_list->timescale,
+                                       e->chapter_list->chapters[i].time);
+      
+      if(chapter_time > e->duration)
         {
         bg_log(BG_LOG_WARNING, LOG_DOMAIN,
                "Omitting chapter %d: time (%f) > duration (%f)",
                i+1,
-               gavl_time_to_seconds(gavl_time_unscale(e->chapter_list->timescale,
-                                                      e->chapter_list->chapters[i].time)),
+               gavl_time_to_seconds(chapter_time),
                gavl_time_to_seconds(e->duration));
         break;
         }
-      if(i < e->chapter_list->num_chapters-1)
+      else
+        num_chapters++;
+      }
+    
+    /* Write chapter information */
+    for(i = 0; i < num_chapters; i++)
+      {
+      if(i < num_chapters-1)
         {
         if(lqt_write_text(e->file, e->chapter_track_id,
                           e->chapter_list->chapters[i].name,
@@ -609,7 +618,8 @@ static void set_audio_parameter_lqt(void * data, int stream, const char * name,
     {
     /* Now we can add the stream */
 
-    e->audio_streams[stream].codec_info = lqt_find_audio_codec_by_name(val->val_str);
+    e->audio_streams[stream].codec_info =
+      lqt_find_audio_codec_by_name(val->val_str);
     
     lqt_gavl_add_audio_track(e->file, &e->audio_streams[stream].format,
                              *e->audio_streams[stream].codec_info);
