@@ -28,17 +28,24 @@
 #include <gmerlin/plugin.h>
 #include <gmerlin/utils.h>
 #include <gmerlin/log.h>
+#include <gmerlin/bggavl.h>
+
+#include "deinterlace.h"
+#include "bgyadif.h"
 
 #define LOG_DOMAIN "fv_deinterlace"
 
-#define DEINTERLACE_NONE     0
-#define DEINTERLACE_GAVL     1
-#define DEINTERLACE_SCALE_HW 2
-#define DEINTERLACE_MJPEGTOOLS 3
+#define DEINTERLACE_NONE       0
+#define DEINTERLACE_GAVL       1
+#define DEINTERLACE_SCALE_HW   2
+#define DEINTERLACE_YADIF      3
+#define DEINTERLACE_YADIF_FAST 4
 
-#ifdef  HAVE_MJPEGTOOLS
-#include <yuvdeinterlace.h>
-#endif
+#define DEINTERLACE_OUTPUT_TOP    0
+#define DEINTERLACE_OUTPUT_BOTTOM 1
+#define DEINTERLACE_OUTPUT_FIRST  2
+#define DEINTERLACE_OUTPUT_SECOND 3
+#define DEINTERLACE_OUTPUT_BOTH   4
 
 typedef struct deinterlace_priv_s
   {
@@ -52,6 +59,7 @@ typedef struct deinterlace_priv_s
   gavl_video_frame_t * frame;
 
   gavl_video_options_t * opt;
+  gavl_video_options_t * global_opt;
 
   gavl_video_deinterlacer_t * deint;
   
@@ -65,12 +73,13 @@ typedef struct deinterlace_priv_s
   int need_restart;
   int need_reinit;
   
+  int output_mode;
+
+  bg_yadif_t * yadif;
+  
   int (*deint_func)(struct deinterlace_priv_s * p,
                    gavl_video_frame_t * frame);
 
-#ifdef HAVE_MJPEGTOOLS
-  yuvdeinterlacer_t * yuvd;
-#endif
   } deinterlace_priv_t;
 
 static void * create_deinterlace()
@@ -79,12 +88,10 @@ static void * create_deinterlace()
   ret = calloc(1, sizeof(*ret));
   ret->deint = gavl_video_deinterlacer_create();
   ret->opt = gavl_video_deinterlacer_get_options(ret->deint);
-
-#ifdef HAVE_MJPEGTOOLS
-  ret->yuvd = yuvdeinterlacer_create();
-#endif
+  ret->yadif = bg_yadif_create();
   
   ret->src_field_1 = gavl_video_frame_create((gavl_video_format_t*)0);
+  ret->global_opt = gavl_video_options_create();
   return ret;
   }
 
@@ -97,59 +104,25 @@ static void destroy_deinterlace(void * priv)
 
   gavl_video_deinterlacer_destroy(vp->deint);
 
+  gavl_video_options_destroy(vp->global_opt);
   gavl_video_frame_null(vp->src_field_1);
   gavl_video_frame_destroy(vp->src_field_1);
-#ifdef HAVE_MJPEGTOOLS
-  yuvdeinterlacer_destroy(vp->yuvd);
-#endif
-  
+  bg_yadif_destroy(vp->yadif);
   free(vp);
   }
 
-
-
-static const bg_parameter_info_t scale_parameters[] =
+static gavl_video_options_t * get_options_deinterlace(void * priv)
   {
-    {
-      .name =        "scale_mode",
-      .long_name =   TRS("Scale mode"),
-      .opt =       "sm",
-      .type =        BG_PARAMETER_STRINGLIST,
-      .flags =     BG_PARAMETER_SYNC,
-      .multi_names = (char const *[]){ "auto",\
-                              "nearest",         \
-                              "bilinear", \
-                              "quadratic", \
-                              "cubic_bspline", \
-                              "cubic_mitchell", \
-                              "cubic_catmull", \
-                              "sinc_lanczos", \
-                              (char*)0 },
-      .multi_labels = (char const *[]){ TRS("Auto"), \
-                               TRS("Nearest"),            \
-                               TRS("Bilinear"), \
-                               TRS("Quadratic"), \
-                               TRS("Cubic B-Spline"), \
-                               TRS("Cubic Mitchell-Netravali"), \
-                               TRS("Cubic Catmull-Rom"), \
-                               TRS("Sinc with Lanczos window"), \
-                               (char*)0 },
-      .val_default = { .val_str = "auto" },
-      .help_string = TRS("Choose scaling method. Auto means to choose based on the conversion quality. Nearest is fastest, Sinc with Lanczos window is slowest."),
-    },
-    {
-      .name =        "scale_order",
-      .long_name =   TRS("Scale order"),
-      .opt =         "so",
-      .type =        BG_PARAMETER_INT,
-      .flags =     BG_PARAMETER_SYNC,
-      .val_min =     { .val_i = 4 },
-      .val_max =     { .val_i = 1000 },
-      .val_default = { .val_i = 4 },
-      .help_string = TRS("Order for sinc scaling"),
-    },
-    { /* */ },
-  };
+  deinterlace_priv_t * vp = priv;
+  return vp->global_opt;
+  }
+
+static void reset_deinterlace(void * priv)
+  {
+  deinterlace_priv_t * vp;
+  vp = (deinterlace_priv_t *)priv;
+  bg_yadif_reset(vp->yadif);
+  }
 
 static const bg_parameter_info_t parameters[] =
   {
@@ -158,7 +131,7 @@ static const bg_parameter_info_t parameters[] =
       .gettext_directory = LOCALE_DIR,
       .name = "method",
       .long_name = TRS("Method"),
-      .type = BG_PARAMETER_MULTI_MENU,
+      .type = BG_PARAMETER_STRINGLIST,
       .flags = BG_PARAMETER_SYNC,
       .val_default = { .val_str = "none" },
       .multi_names =  (char const *[]){ "none",
@@ -166,11 +139,8 @@ static const bg_parameter_info_t parameters[] =
                                "scale_hw",
                                "scale_sw",
                                "blend",
-#ifdef HAVE_MJPEGTOOLS
-                               "yuvdeinterlace_1",
-                               "yuvdeinterlace_2",
-                               "yuvdeinterlace_aa",
-#endif                               
+                               "yadif",
+                               "yadif_fast",
                                (char*)0 },
       
       .multi_labels = (char const *[]){ TRS("None"),
@@ -178,46 +148,17 @@ static const bg_parameter_info_t parameters[] =
                                TRS("Scaler (hardware)"),
                                TRS("Scaler (software)"),
                                TRS("Blend"),
-#ifdef HAVE_MJPEGTOOLS
-                               TRS("yuvdeinterlace (1 field)"),
-                               TRS("yuvdeinterlace (2 fields)"),
-                               TRS("yuvdeinterlace (Antialiasing only)"),
-#endif
+                               TRS("Yadif"),
+                               TRS("Yadif (fast)"),
                                (char*)0 },
       .multi_descriptions = (char const *[]){ TRS("Do nothing"),
                                      TRS("Simply double all scanlines. Very fast but \
 low image quality"), 
                                      TRS("Drop one field and change the pixel aspect ratio such that a subsequent hardware scaler will scale the image to the original height"),
                                      TRS("Drop one field and scale the image to the original height"),
-#ifdef HAVE_MJPEGTOOLS
-                                     TRS("Motion compensating deinterlacer from the mjpegtools project"),
-                                     TRS("Motion compensating deinterlacer from the mjpegtools project"),
-                                     TRS("Motion compensating deinterlacer from the mjpegtools project"),
-#endif                               
+                                     TRS("yadif"),
+                                     TRS("yadif (fast mode)"),
                                      (char*)0 },
-      .multi_parameters = (bg_parameter_info_t const *[]) { (bg_parameter_info_t*)0, // None
-                                                   (bg_parameter_info_t*)0, // Copy
-                                                   (bg_parameter_info_t*)0, // Scale (HW)
-                                                   scale_parameters, // Scale (SW)
-                                                   (bg_parameter_info_t*)0, // Blend
-#ifdef HAVE_MJPEGTOOLS
-                                                   (bg_parameter_info_t*)0, // yuvdeinterlace
-                                                   (bg_parameter_info_t*)0, // yuvdeinterlace
-                                                   (bg_parameter_info_t*)0, // yuvdeinterlace
-#endif
-                                                   (bg_parameter_info_t*)0 } 
-                                                   
-    },
-    {
-      .name =        "quality",
-      .long_name =   TRS("Quality"),
-      .opt =         "q",
-      .type =        BG_PARAMETER_SLIDER_INT,
-      .flags =     BG_PARAMETER_SYNC,
-      .val_min =     { .val_i = GAVL_QUALITY_FASTEST },
-      .val_max =     { .val_i = GAVL_QUALITY_BEST    },
-      .val_default = { .val_i = GAVL_QUALITY_DEFAULT },
-      .help_string = TRS("Lower quality means more speed. Values above 3 enable slow high quality calculations.")
     },
     {
       .name = "force",
@@ -227,17 +168,21 @@ low image quality"),
       .help_string = "Always perform deinterlacing even if the source format pretends to be progressive",
     },
     {
-      .name = "drop_mode",
-      .long_name = TRS("Drop mode"),
+      .name = "output_mode",
+      .long_name = TRS("Output mode"),
       .type = BG_PARAMETER_STRINGLIST,
       .flags = BG_PARAMETER_SYNC,
       .val_default = { .val_str = "top" },
-      .help_string = TRS("For modes, which take only one source field, specify which field to drop"),
-      .multi_names = (char const *[]){ "top", "bottom", (char*)0 },
-      .multi_labels = (char const *[]){ TRS("Drop top field"),
-                               TRS("Drop bottom field"),
-                               (char*)0 },
+      .help_string = TRS("Specify which field to output. Outputting both fields is not always supported."),
+      .multi_names = (char const *[]){ "top", "bottom", "first", "second", "both", (char*)0 },
+      .multi_labels = (char const *[]){ TRS("Top field"),
+                                        TRS("Bottom field"),
+                                        TRS("First field"),
+                                        TRS("Second field"),
+                                        TRS("Both fields"),
+                                        (char*)0 },
     },
+    BG_GAVL_PARAM_SCALE_MODE,
     { /* End of parameters */ },
   };
 
@@ -246,45 +191,17 @@ static const bg_parameter_info_t * get_parameters_deinterlace(void * priv)
   return parameters;
   }
 
-
-static gavl_scale_mode_t string_to_scale_mode(const char * str)
-  {
-  if(!strcmp(str, "auto"))
-    return GAVL_SCALE_AUTO;
-  else if(!strcmp(str, "nearest"))
-    return GAVL_SCALE_NEAREST;
-  else if(!strcmp(str, "bilinear"))
-    return GAVL_SCALE_BILINEAR;
-  else if(!strcmp(str, "quadratic"))
-    return GAVL_SCALE_QUADRATIC;
-  else if(!strcmp(str, "cubic_bspline"))
-    return GAVL_SCALE_CUBIC_BSPLINE;
-  else if(!strcmp(str, "cubic_mitchell"))
-    return GAVL_SCALE_CUBIC_MITCHELL;
-  else if(!strcmp(str, "cubic_catmull"))
-    return GAVL_SCALE_CUBIC_CATMULL;
-  else if(!strcmp(str, "sinc_lanczos"))
-    return GAVL_SCALE_SINC_LANCZOS;
-  else
-    {
-    bg_log(BG_LOG_ERROR, LOG_DOMAIN, "Unknown scale mode %s", str);
-    return GAVL_SCALE_AUTO;
-    }
-  }
-
 static void set_parameter_deinterlace(void * priv, const char * name,
                                       const bg_parameter_value_t * val)
   {
   int new_method = DEINTERLACE_NONE;
   gavl_deinterlace_mode_t new_sub_method = 0;
-  gavl_deinterlace_drop_mode_t new_drop_mode;
   gavl_scale_mode_t new_scale_mode;
+  int new_output_mode;
   
   deinterlace_priv_t * vp;
-  int new_src_field = 0;
   vp = (deinterlace_priv_t *)priv;
   
-  
   if(!name)
     return;
 
@@ -313,23 +230,16 @@ static void set_parameter_deinterlace(void * priv, const char * name,
       new_method = DEINTERLACE_GAVL;
       new_sub_method = GAVL_DEINTERLACE_BLEND;
       }
-#ifdef HAVE_MJPEGTOOLS
-    else if(!strcmp(val->val_str, "yuvdeinterlace_1"))
+    else if(!strcmp(val->val_str, "yadif"))
       {
-      new_method = DEINTERLACE_MJPEGTOOLS;
-      new_sub_method = YUVD_MODE_DEINT_1;
+      new_method = DEINTERLACE_YADIF;
+      // new_sub_method = GAVL_DEINTERLACE_BLEND;
       }
-    else if(!strcmp(val->val_str, "yuvdeinterlace_2"))
+    else if(!strcmp(val->val_str, "yadif_fast"))
       {
-      new_method = DEINTERLACE_MJPEGTOOLS;
-      new_sub_method = YUVD_MODE_DEINT_2;
+      new_method = DEINTERLACE_YADIF_FAST;
+      // new_sub_method = GAVL_DEINTERLACE_BLEND;
       }
-    else if(!strcmp(val->val_str, "yuvdeinterlace_aa"))
-      {
-      new_method = DEINTERLACE_MJPEGTOOLS;
-      new_sub_method = YUVD_MODE_ANTIALIAS;
-      }
-#endif
     if((new_method != vp->method) || 
        (new_sub_method != vp->sub_method))
       {
@@ -342,10 +252,6 @@ static void set_parameter_deinterlace(void * priv, const char * name,
         gavl_video_options_set_deinterlace_mode(vp->opt,
                                                 new_sub_method);
         }
-#ifdef HAVE_MJPEGTOOLS
-      else if(new_method == DEINTERLACE_MJPEGTOOLS)
-        yuvdeinterlacer_set_mode(vp->yuvd, new_sub_method);
-#endif
       }
     }
   else if(!strcmp(name, "force"))
@@ -356,28 +262,28 @@ static void set_parameter_deinterlace(void * priv, const char * name,
       vp->need_restart = 1;
       }
     }
-  else if(!strcmp(name, "drop_mode"))
+  else if(!strcmp(name, "output_mode"))
     {
     if(!strcmp(val->val_str, "top"))
+      new_output_mode = DEINTERLACE_OUTPUT_TOP;
+    else if(!strcmp(val->val_str, "bottom"))
+      new_output_mode = DEINTERLACE_OUTPUT_BOTTOM;
+    else if(!strcmp(val->val_str, "first"))
+      new_output_mode = DEINTERLACE_OUTPUT_FIRST;
+    else if(!strcmp(val->val_str, "second"))
+      new_output_mode = DEINTERLACE_OUTPUT_SECOND;
+    else if(!strcmp(val->val_str, "both"))
+      new_output_mode = DEINTERLACE_OUTPUT_BOTH;
+    
+    if(new_output_mode != vp->output_mode)
       {
-      new_drop_mode = GAVL_DEINTERLACE_DROP_TOP;
-      new_src_field = 1;
-      }
-    else
-      {
-      new_drop_mode = GAVL_DEINTERLACE_DROP_BOTTOM;
-      new_src_field = 0;
-      }
-    if(new_drop_mode != gavl_video_options_get_deinterlace_drop_mode(vp->opt))
-      {
-      gavl_video_options_set_deinterlace_drop_mode(vp->opt, new_drop_mode);
       vp->need_restart = 1;
+      vp->output_mode = new_output_mode;
       }
-    vp->src_field = new_src_field;
     }
   else if(!strcmp(name, "scale_mode"))
     {
-    new_scale_mode = string_to_scale_mode(val->val_str);
+    new_scale_mode = bg_gavl_string_to_scale_mode(val->val_str);
     if(new_scale_mode != gavl_video_options_get_scale_mode(vp->opt))
       {
       gavl_video_options_set_scale_mode(vp->opt, new_scale_mode);
@@ -392,17 +298,8 @@ static void set_parameter_deinterlace(void * priv, const char * name,
       vp->need_reinit = 1;
       }
     }
-  else if(!strcmp(name, "quality"))
-    {
-    if(gavl_video_options_get_quality(vp->opt) != val->val_i)
-      {
-      gavl_video_options_set_quality(vp->opt, val->val_i);
-      vp->need_reinit = 1;
-      }
-    }
   }
 
-
 /*
  * Deinterlace functions
  */
@@ -453,13 +350,11 @@ static int deinterlace_scale_hw(struct deinterlace_priv_s * vp,
   return 1;
   }
 
-#ifdef HAVE_MJPEGTOOLS
-static int deinterlace_mjpegtools(struct deinterlace_priv_s * vp,
-                                   gavl_video_frame_t * frame)
+static int deinterlace_yadif(struct deinterlace_priv_s * vp,
+                             gavl_video_frame_t * frame)
   {
-  return yuvdeinterlacer_read(vp->yuvd, frame, 0);
+  return bg_yadif_read(vp->yadif, frame, 0);
   }
-#endif
 
 static void connect_input_port_deinterlace(void * priv,
                                            bg_read_video_func_t func,
@@ -474,21 +369,37 @@ static void connect_input_port_deinterlace(void * priv,
     vp->read_func = func;
     vp->read_data = data;
     vp->read_stream = stream;
-#ifdef HAVE_MJPEGTOOLS
-    yuvdeinterlacer_connect_input(vp->yuvd, vp->read_func, vp->read_data, vp->read_stream);
-#endif
-
+    bg_yadif_connect_input(vp->yadif, func, data, stream);
     }
   }
 
+static void transfer_global_options(gavl_video_options_t * opt,
+                                    gavl_video_options_t * global_opt)
+  {
+  void * client_data;
+  gavl_video_stop_func stop_func;
+  gavl_video_run_func  run_func;
+  
+  gavl_video_options_set_quality(opt, gavl_video_options_get_quality(global_opt));
+  gavl_video_options_set_num_threads(opt, gavl_video_options_get_num_threads(global_opt));
+                                     
+  run_func = gavl_video_options_get_run_func(global_opt, &client_data);
+  gavl_video_options_set_run_func(opt, run_func, client_data);
+
+  stop_func = gavl_video_options_get_stop_func(global_opt, &client_data);
+  gavl_video_options_set_stop_func(opt, stop_func, client_data);
+  
+  }
+
+
 static void
 set_input_format_deinterlace(void * priv,
                              gavl_video_format_t * format,
                              int port)
   {
   deinterlace_priv_t * vp;
+  int yadif_mode;
   vp = (deinterlace_priv_t *)priv;
-  
   if(!port)
     {
     if(vp->frame)
@@ -496,9 +407,7 @@ set_input_format_deinterlace(void * priv,
       gavl_video_frame_destroy(vp->frame);
       vp->frame = (gavl_video_frame_t*)0;
       }
-    gavl_video_format_copy(&vp->in_format, format);
-    gavl_video_format_copy(&vp->out_format, format);
-    vp->out_format.interlace_mode = GAVL_INTERLACE_NONE;
+    transfer_global_options(vp->opt, vp->global_opt);
     
     vp->need_reinit = 1;
     
@@ -506,9 +415,15 @@ set_input_format_deinterlace(void * priv,
       {
       case DEINTERLACE_NONE:
         vp->deint_func = deinterlace_none;
+        gavl_video_format_copy(&vp->in_format, format);
+        gavl_video_format_copy(&vp->out_format, format);
+        vp->out_format.interlace_mode = GAVL_INTERLACE_NONE;
         break;
       case DEINTERLACE_GAVL:
         vp->deint_func = deinterlace_gavl;
+        gavl_video_format_copy(&vp->in_format, format);
+        gavl_video_format_copy(&vp->out_format, format);
+        vp->out_format.interlace_mode = GAVL_INTERLACE_NONE;
         break;
       case DEINTERLACE_SCALE_HW:
         vp->deint_func = deinterlace_scale_hw;
@@ -516,18 +431,26 @@ set_input_format_deinterlace(void * priv,
         vp->out_format.image_height /= 2;
         vp->out_format.frame_height /= 2;
         vp->out_format.pixel_height *= 2;
-        break;
-#ifdef HAVE_MJPEGTOOLS
-      case DEINTERLACE_MJPEGTOOLS:
-        vp->deint_func = deinterlace_mjpegtools;
-        yuvdeinterlacer_init(vp->yuvd, &vp->in_format);
-        gavl_video_format_copy(format, &vp->in_format);
-        yuvdeinterlacer_get_output_format(vp->yuvd, &vp->out_format);
 
-        
+        gavl_video_format_copy(&vp->in_format, format);
+        gavl_video_format_copy(&vp->out_format, format);
+        vp->out_format.interlace_mode = GAVL_INTERLACE_NONE;
         
         break;
-#endif
+      case DEINTERLACE_YADIF:
+      case DEINTERLACE_YADIF_FAST:
+        vp->deint_func = deinterlace_yadif;
+
+        yadif_mode = (vp->output_mode == DEINTERLACE_OUTPUT_BOTH) ?
+          1 : 0;
+
+        if(vp->method == DEINTERLACE_YADIF_FAST)
+          yadif_mode += 2;
+        
+        bg_yadif_init(vp->yadif, format, vp->opt, yadif_mode);
+        gavl_video_format_copy(&vp->in_format, format);
+        bg_yadif_get_output_format(vp->yadif, &vp->out_format);
+        break;
       }
     
     vp->need_restart = 0;
@@ -565,6 +488,7 @@ static int read_video_deinterlace(void * priv, gavl_video_frame_t * frame, int s
     switch(vp->method)
       {
       case DEINTERLACE_GAVL:
+        transfer_global_options(vp->opt, vp->global_opt);
         gavl_video_deinterlacer_init(vp->deint, &vp->in_format);
         break;
       default:
@@ -591,6 +515,7 @@ const const bg_fv_plugin_t the_plugin =
       .set_parameter =    set_parameter_deinterlace,
       .priority =         1,
     },
+    .get_options = get_options_deinterlace,
     
     .connect_input_port = connect_input_port_deinterlace,
     
@@ -599,6 +524,7 @@ const const bg_fv_plugin_t the_plugin =
     
     .read_video = read_video_deinterlace,
     .need_restart = need_restart_deinterlace,
+    .reset = reset_deinterlace,
   };
 
 /* Include this into all plugin modules exactly once
