@@ -53,9 +53,9 @@ typedef struct
   gavl_video_frame_t * frame;
   gavl_image_transform_t * transform;
   gavl_video_options_t * opt;
+  gavl_video_options_t * global_opt;
   
   int changed;
-  int quality;
   int scale_order;
   gavl_scale_mode_t scale_mode;
 
@@ -109,15 +109,41 @@ typedef struct
   
   } transform_t;
 
+static void transfer_global_options(gavl_video_options_t * opt,
+                                    gavl_video_options_t * global_opt)
+  {
+  void * client_data;
+  gavl_video_stop_func stop_func;
+  gavl_video_run_func  run_func;
+  
+  gavl_video_options_set_quality(opt, gavl_video_options_get_quality(global_opt));
+  gavl_video_options_set_num_threads(opt, gavl_video_options_get_num_threads(global_opt));
+                                     
+  run_func = gavl_video_options_get_run_func(global_opt, &client_data);
+  gavl_video_options_set_run_func(opt, run_func, client_data);
+
+  stop_func = gavl_video_options_get_stop_func(global_opt, &client_data);
+  gavl_video_options_set_stop_func(opt, stop_func, client_data);
+  
+  }
+
 static void * create_transform()
   {
   transform_t * ret;
   ret = calloc(1, sizeof(*ret));
   ret->transform = gavl_image_transform_create();
   ret->opt = gavl_image_transform_get_options(ret->transform);
+  ret->global_opt = gavl_video_options_create();
   return ret;
   }
 
+static gavl_video_options_t * get_options_transform(void * priv)
+  {
+  transform_t * vp = priv;
+  return vp->global_opt;
+  }
+
+
 static void destroy_transform(void * priv)
   {
   transform_t * vp;
@@ -126,6 +152,7 @@ static void destroy_transform(void * priv)
     gavl_video_frame_destroy(vp->frame);
 
   gavl_image_transform_destroy(vp->transform);
+  gavl_video_options_destroy(vp->global_opt);
   
   free(vp);
   }
@@ -169,15 +196,6 @@ static const bg_parameter_info_t parameters[] =
     .val_default = { .val_str = "auto" },
     .help_string = TRS("Choose interpolation method. Auto means to choose based on the conversion quality. Nearest is fastest, Bicubic is slowest."),
     },
-    {
-      .name = "quality",
-      .long_name = TRS("Quality"),
-      .type = BG_PARAMETER_SLIDER_INT,
-      .flags = BG_PARAMETER_SYNC,
-      .val_min =     { .val_i = GAVL_QUALITY_FASTEST },
-      .val_max =     { .val_i = GAVL_QUALITY_BEST },
-      .val_default = { .val_i = GAVL_QUALITY_DEFAULT },
-    },
     {
       .name = "bg_color",
       .long_name = TRS("Background color"),
@@ -416,14 +434,6 @@ set_parameter_transform(void * priv, const char * name,
       vp->changed = 1;
       }
     }
-  else if(!strcmp(name, "quality"))
-    {
-    if(vp->quality != val->val_i)
-      {
-      vp->quality = val->val_i;
-      vp->changed = 1;
-      }
-    }
   else if(!strcmp(name, "bg_color"))
     {
     memcpy(vp->bg_color, val->val_color, sizeof(vp->bg_color));
@@ -745,8 +755,7 @@ static void init_matrix_rotate(transform_t * vp)
   
   sin_angle = sin(vp->rotate_angle / 180.0 * M_PI);
   cos_angle = cos(vp->rotate_angle / 180.0 * M_PI);
-
-
+  
   mat[0][0] = cos_angle;
   mat[1][0] = sin_angle;
   mat[0][1] = -sin_angle;
@@ -1204,7 +1213,8 @@ static void init_transform(transform_t * vp)
     return;
   
   gavl_video_options_set_scale_mode(vp->opt, vp->scale_mode);
-  gavl_video_options_set_quality(vp->opt, vp->quality);
+
+  transfer_global_options(vp->opt, vp->global_opt);
   
   gavl_image_transform_init(vp->transform, &vp->format,
                             func, vp);
@@ -1261,6 +1271,7 @@ const bg_fv_plugin_t the_plugin =
     },
     
     .connect_input_port = connect_input_port_transform,
+    .get_options        = get_options_transform,
     
     .set_input_format = set_input_format_transform,
     .get_output_format = get_output_format_transform,
